@@ -28,7 +28,7 @@ try:
 except ImportError:
     logger.debug('cannot import pygame tools')
 
-__version__ = (3, 22, 1)
+__version__ = (3, 25)
 __author__ = 'bitcraft'
 __author_email__ = 'leif.theden@gmail.com'
-__description__ = 'Map loader for TMX Files - Python 2 and 3'
+__description__ = 'Map loader for TMX Files - Python 3.3 +'
