@@ -17,20 +17,15 @@ GNU Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with pytmx.  If not, see <http://www.gnu.org/licenses/>.
 """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import logging
 import os
 from collections import defaultdict, namedtuple
+from io import BytesIO
 from itertools import chain, product
+from math import cos, radians, sin
 from operator import attrgetter
 from xml.etree import ElementTree
 
-import six
-from six.moves import map
-
 __all__ = (
     'TiledElement',
     'TiledMap',
@@ -63,8 +58,9 @@ flag_names = (
     'flipped_vertically',
     'flipped_diagonally')
 
-TileFlags = namedtuple('TileFlags', flag_names)
 AnimationFrame = namedtuple('AnimationFrame', ['gid', 'duration'])
+Point = namedtuple("Point", ["x", "y"])
+TileFlags = namedtuple('TileFlags', flag_names)
 
 
 def default_image_loader(filename, flags, **kwargs):
@@ -113,15 +109,27 @@ def convert_to_bool(value):
     raise ValueError('cannot parse "{}" as bool'.format(value))
 
 
+def rotate(points, origin, angle):
+    sin_t = sin(radians(angle))
+    cos_t = cos(radians(angle))
+    new_points = list()
+    for point in points:
+        p = (
+            origin.x + (cos_t * (point.x - origin.x) - sin_t * (point.y - origin.y)),
+            origin.y + (sin_t * (point.x - origin.x) + cos_t * (point.y - origin.y)),
+        )
+        new_points.append(p)
+    return new_points
+
+
 # used to change the unicode string returned from xml to
 # proper python variable types.
-types = defaultdict(lambda: six.u)
+types = defaultdict(lambda: str)
 
-_str = six.u
 types.update({
     "version": str,
     "tiledversion": str,
-    "orientation": _str,
+    "orientation": str,
     "renderorder": str,
     "width": float,
     "height": float,
@@ -133,14 +141,14 @@ types.update({
     "backgroundcolor": str,
     "nextobjectid": int,
     "firstgid": int,
-    "source": _str,
-    "name": _str,
+    "source": str,
+    "name": str,
     "spacing": int,
     "margin": int,
     "tilecount": int,
     "columns": int,
     "format": str,
-    "trans": _str,
+    "trans": str,
     "tile": int,
     "terrain": str,
     "probability": float,
@@ -152,8 +160,8 @@ types.update({
     "visible": convert_to_bool,
     "offsetx": int,
     "offsety": int,
-    "encoding": _str,
-    "compression": _str,
+    "encoding": str,
+    "compression": str,
     "draworder": str,
     "points": str,
     "fontfamily": str,
@@ -167,10 +175,10 @@ types.update({
     "halign": str,
     "valign": str,
     "gid": int,
-    "type": _str,
+    "type": str,
     "x": float,
     "y": float,
-    "value": _str,
+    "value": str,
     "rotation": float,
 })
 
@@ -181,7 +189,8 @@ prop_type = {
     'float': float,
     'bool': convert_to_bool,
     'color': str,
-    'file': str
+    'file': str,
+    'object': int,
 }
 
 
@@ -200,7 +209,9 @@ def parse_properties(node):
                     cls = prop_type[subnode.get("type")]
             except AttributeError:
                 logger.info("Type {} Not a built-in type. Defaulting to string-cast.".format(subnode.get("type")))
-            d[subnode.get('name')] = cls(subnode.get('value')) if cls is not None else subnode.get('value')
+            d[subnode.get('name')] = subnode.get('value') or subnode.text
+            if cls is not None:
+                d[subnode.get('name')] = cls(subnode.get('value'))
     return d
 
 
@@ -279,7 +290,10 @@ class TiledElement(object):
                 raise AttributeError("Element has no property {0}".format(item))
 
     def __repr__(self):
-        return '<{0}: "{1}">'.format(self.__class__.__name__, self.name)
+        if hasattr(self, "id"):
+            return '<{}[{}]: "{}">'.format(self.__class__.__name__, self.id, self.name)
+        else:
+            return '<{}: "{}">'.format(self.__class__.__name__, self.name)
 
 
 class TiledMap(TiledElement):
@@ -320,6 +334,8 @@ class TiledMap(TiledElement):
         self.tilesets = list()  # TiledTileset objects
         self.tile_properties = dict()  # tiles that have properties
         self.layernames = dict()
+        self.objects_by_id = dict()
+        self.objects_by_name = dict()
 
         # only used tiles are actually loaded, so there will be a difference
         # between the GIDs in the Tiled map data (tmx) and the data in this
@@ -387,13 +403,18 @@ class TiledMap(TiledElement):
         for subnode in node.findall('imagelayer'):
             self.add_layer(TiledImageLayer(self, subnode))
 
+        # this will only find objectgroup layers, not including tile colliders
         for subnode in node.findall('objectgroup'):
-            self.add_layer(TiledObjectGroup(self, subnode))
+            objectgroup = TiledObjectGroup(self, subnode)
+            self.add_layer(objectgroup)
+            for obj in objectgroup:
+                self.objects_by_id[obj.id] = obj
+                self.objects_by_name[obj.name] = obj
 
         for subnode in node.findall('tileset'):
             self.add_tileset(TiledTileset(self, subnode))
 
-        # "tile objects", objects with a GID, have need to have their attributes
+        # "tile objects", objects with a GID, require their attributes to be
         # set after the tileset is loaded, so this step must be performed last
         # also, this step is performed for objects to load their tiles.
 
@@ -673,16 +694,21 @@ class TiledMap(TiledElement):
             logger.debug(msg.format(name))
             raise ValueError(msg.format(name))
 
+    def get_object_by_id(self, obj_id):
+        """Find an object
+
+        :param name: Name of object.  Case-sensitive.
+        :rtype: Object if found, otherwise ValueError
+        """
+        return self.objects_by_id[obj_id]
+
     def get_object_by_name(self, name):
         """Find an object
 
         :param name: Name of object.  Case-sensitive.
         :rtype: Object if found, otherwise ValueError
         """
-        for obj in self.objects:
-            if obj.name == name:
-                return obj
-        raise ValueError("Object {0} not found".format(name))
+        return self.objects_by_name[name]
 
     def get_tileset_from_gid(self, gid):
         """ Return tileset that owns the gid
@@ -705,6 +731,13 @@ class TiledMap(TiledElement):
 
         raise ValueError("Tileset not found")
 
+    def get_tile_colliders(self):
+        """Return iterator of (gid, dict) pairs of tiles with colliders"""
+        for gid, props in self.tile_properties.items():
+            colliders = props.get("colliders")
+            if colliders:
+                yield gid, colliders
+
     @property
     def objectgroups(self):
         """Return iterator of all object groups
@@ -909,6 +942,10 @@ class TiledTileset(TiledElement):
                     gid = register_gid(int(frame.get('tileid')) + self.firstgid)
                     frames.append(AnimationFrame(gid, duration))
 
+            for objgrp_node in child.findall("objectgroup"):
+                objectgroup = TiledObjectGroup(self.parent, objgrp_node)
+                p["colliders"] = objectgroup
+
             for gid, flags in self.parent.map_gid2(tiled_gid + self.firstgid):
                 self.parent.set_tile_properties(gid, p)
 
@@ -1030,7 +1067,7 @@ class TiledTileLayer(TiledElement):
         if compression == 'gzip':
             import gzip
 
-            with gzip.GzipFile(fileobj=six.BytesIO(data)) as fh:
+            with gzip.GzipFile(fileobj=BytesIO(data)) as fh:
                 data = fh.read()
 
         elif compression == 'zlib':
@@ -1091,7 +1128,7 @@ class TiledObjectGroup(TiledElement, list):
         self.visible = 1
         self.offsetx = 0
         self.offsety = 0
-        self.draworder = "topdown"
+        self.draworder = "index"
 
         self.parse_xml(node)
 
@@ -1129,6 +1166,7 @@ class TiledObject(TiledElement):
         self.rotation = 0
         self.gid = 0
         self.visible = 1
+        self.closed = True
         self.template = None
 
         self.parse_xml(node)
@@ -1178,10 +1216,29 @@ class TiledObject(TiledElement):
             self.width = abs(x1) + abs(x2)
             self.height = abs(y1) + abs(y2)
             self.points = tuple(
-                [(i[0] + self.x, i[1] + self.y) for i in points])
+                [Point(i[0] + self.x, i[1] + self.y) for i in points])
 
         return self
 
+    def apply_transformations(self):
+        """Return all points for object, taking in account rotation"""
+        if hasattr(self, "points"):
+            return rotate(self.points, self, self.rotation)
+        else:
+            return rotate(self.as_points, self, self.rotation)
+
+    @property
+    def as_points(self):
+        return [
+            Point(*i)
+            for i in [
+                (self.x, self.y),
+                (self.x, self.y + self.height),
+                (self.x + self.width, self.y + self.height),
+                (self.x + self.width, self.y),
+            ]
+        ]
+
 
 class TiledImageLayer(TiledElement):
     """ Represents Tiled Image Layer
